@@ -97,6 +97,7 @@
 #define bit_unfmt		slurm_bit_unfmt
 #define	bitfmt2int		slurm_bitfmt2int
 #define	bit_fmt_hexmask		slurm_bit_fmt_hexmask
+#define	bit_fmt_hexmask_trim	slurm_bit_fmt_hexmask_trim
 #define bit_unfmt_hexmask	slurm_bit_unfmt_hexmask
 #define	bit_fmt_binmask		slurm_bit_fmt_binmask
 #define bit_unfmt_binmask	slurm_bit_unfmt_binmask
@@ -186,6 +187,7 @@
 #define	list_prepend		slurm_list_prepend
 #define	list_find_first		slurm_list_find_first
 #define	list_delete_all		slurm_list_delete_all
+#define	list_delete_ptr		slurm_list_delete_ptr
 #define	list_for_each		slurm_list_for_each
 #define	list_for_each_max	slurm_list_for_each_max
 #define	list_sort		slurm_list_sort
@@ -219,8 +221,6 @@
 #define	fatal_abort		slurm_fatal_abort
 #define	fatal			slurm_fatal
 #define	error			slurm_error
-#define	info			slurm_info
-#define	verbose			slurm_verbose
 
 /* macros.h functions
  * None exported today.
@@ -228,7 +228,6 @@
 
 /* net.[ch] functions */
 #define net_stream_listen	slurm_net_stream_listen
-#define net_set_low_water	slurm_net_set_low_water
 
 /* node_conf.[ch] functions */
 #define init_node_conf          slurm_init_node_conf
@@ -302,7 +301,7 @@
 #define running_in_slurmctld    slurm_running_in_slurmctld
 #define running_in_slurmd       slurm_running_in_slurmd
 #define running_in_slurmdbd     slurm_running_in_slurmdbd
-#define running_in_slurmdstepd  slurm_running_in_slurmdstepd
+#define running_in_slurmd_stepd slurm_running_in_slurmd_stepd
 #define running_in_slurmstepd   slurm_running_in_slurmstepd
 
 /* slurm_auth.[ch] functions
@@ -322,6 +321,7 @@
 #define	__xassert_failed	slurm_xassert_failed
 
 /* xmalloc.[ch] functions */
+#define xsize			slurm_xsize
 #define xfree_ptr		slurm_xfree_ptr
 
 /* xsignal.[ch] functions */
@@ -453,7 +453,6 @@
 /* some stepd_api.[ch] functions */
 #define stepd_available			slurm_stepd_available
 #define stepd_connect			slurm_stepd_connect
-#define stepd_connect_nss		slurm_stepd_connect_nss
 #define stepd_get_uid			slurm_stepd_get_uid
 #define stepd_add_extern_pid		slurm_stepd_add_extern_pid
 #define stepd_get_x11_display		slurm_stepd_get_x11_display
@@ -462,6 +461,7 @@
 #define xfree_struct_passwd		slurm_xfree_struct_passwd
 #define stepd_getgr			slurm_stepd_getgr
 #define xfree_struct_group_array	slurm_xfree_struct_group_array
+#define stepd_get_namespace_fd		slurm_stepd_get_namespace_fd
 
 /* xcgroup_read_config.[ch] */
 #define xcgroup_config_read_mutex       slurm_xcgroup_config_read_mutex
