@@ -71,7 +71,6 @@
 
 strong_alias(stepd_available, slurm_stepd_available);
 strong_alias(stepd_connect, slurm_stepd_connect);
-strong_alias(stepd_connect_nss, slurm_stepd_connect_nss);
 strong_alias(stepd_get_uid, slurm_stepd_get_uid);
 strong_alias(stepd_add_extern_pid, slurm_stepd_add_extern_pid);
 strong_alias(stepd_get_x11_display, slurm_stepd_get_x11_display);
@@ -80,21 +79,7 @@ strong_alias(stepd_getpw, slurm_stepd_getpw);
 strong_alias(xfree_struct_passwd, slurm_xfree_struct_passwd);
 strong_alias(stepd_getgr, slurm_stepd_getgr);
 strong_alias(xfree_struct_group_array, slurm_xfree_struct_group_array);
-
-static bool
-_slurm_authorized_user()
-{
-	uid_t uid, slurm_user_id;
-	slurm_ctl_conf_t *conf;
-
-	conf = slurm_conf_lock();
-	slurm_user_id = (uid_t)conf->slurm_user_id;
-	slurm_conf_unlock();
-
-	uid = getuid();
-
-	return ((uid == (uid_t)0) || (uid == slurm_user_id));
-}
+strong_alias(stepd_get_namespace_fd, slurm_stepd_get_namespace_fd);
 
 /*
  * Should be called when a connect() to a socket returns ECONNREFUSED.
@@ -110,8 +95,8 @@ _handle_stray_socket(const char *socket_name)
 	time_t now;
 
 	/* Only attempt to remove the stale socket if process is running
-	   as root or the SlurmUser. */
-	if (!_slurm_authorized_user())
+	   as root or the SlurmdUser. */
+	if (getuid() && (getuid() != slurm_conf.slurmd_user_id))
 		return;
 
 	if (stat(socket_name, &buf) == -1) {
@@ -156,13 +141,20 @@ static void _handle_stray_script(const char *directory, uint32_t job_id)
 
 static int
 _step_connect(const char *directory, const char *nodename,
-	      uint32_t jobid, uint32_t stepid)
+	      slurm_step_id_t *step_id)
 {
 	int fd;
 	int len;
 	struct sockaddr_un addr;
-	char *name = xstrdup_printf("%s/%s_%u.%u",
-				    directory, nodename, jobid, stepid);
+	char *name = NULL, *pos = NULL;
+	uint32_t stepid = step_id->step_id;
+	bool old_id_tied = false;
+
+try_old_id:
+	xstrfmtcatat(name, &pos, "%s/%s_%u.%u",
+		     directory, nodename, step_id->job_id, stepid);
+	if (step_id->step_het_comp != NO_VAL)
+		xstrfmtcatat(name, &pos, ".%u", step_id->step_het_comp);
 
 	/*
 	 * If socket name would be truncated, emit error and exit
@@ -176,8 +168,8 @@ _step_connect(const char *directory, const char *nodename,
 	}
 
 	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
-		error("%s: socket() failed dir %s node %s job %u step %u %m",
-		      __func__, directory, nodename, jobid, stepid);
+		error("%s: socket() failed for %s: %m",
+		      __func__, name);
 		xfree(name);
 		return -1;
 	}
@@ -189,13 +181,34 @@ _step_connect(const char *directory, const char *nodename,
 
 	if (connect(fd, (struct sockaddr *) &addr, len) < 0) {
 		/* Can indicate race condition at step termination */
-		debug("%s: connect() failed dir %s node %s step %u.%u %m",
-		      __func__, directory, nodename, jobid, stepid);
+		debug("%s: connect() failed for %s: %m",
+		      __func__, name);
 		if (errno == ECONNREFUSED && running_in_slurmd()) {
 			_handle_stray_socket(name);
-			if (stepid == SLURM_BATCH_SCRIPT)
-				_handle_stray_script(directory, jobid);
+			/*
+			 * NOTE: Checking against NO_VAL can be removed after 21.08
+			 */
+			if ((step_id->step_id == SLURM_BATCH_SCRIPT) ||
+			    (step_id->step_id == NO_VAL))
+				_handle_stray_script(directory,
+						     step_id->job_id);
 		}
+
+		/* NOTE: This code can be removed after 21.08 */
+		if (errno == ENOENT && !old_id_tied &&
+		    ((step_id->step_id == SLURM_BATCH_SCRIPT) ||
+		     (step_id->step_id == SLURM_EXTERN_CONT))) {
+			debug("%s: Try to use old step_id", __func__);
+			close(fd);
+			if (stepid == SLURM_BATCH_SCRIPT)
+				stepid = NO_VAL;
+			else
+				stepid = INFINITE;
+			pos = name;
+			old_id_tied = true;
+			goto try_old_id;
+		}
+
 		xfree(name);
 		close(fd);
 		return -1;
@@ -224,78 +237,6 @@ _guess_nodename(void)
 	return nodename;
 }
 
-/*
- * Legacy version for connecting to pre-19.05 stepds.
- * Remove this two versions after 19.05 is released.
- */
-static int _stepd_connect_legacy(const char *directory, const char *nodename,
-				 uint32_t jobid, uint32_t stepid,
-				 uint16_t *protocol_version)
-{
-	int req = REQUEST_CONNECT;
-	int fd = -1;
-	int rc;
-	void *auth_cred;
-	char *auth_info;
-	char *local_nodename = NULL;
-	Buf buffer;
-	int len;
-
-	buffer = init_buf(0);
-	/* Create an auth credential */
-	auth_info = slurm_get_auth_info();
-	auth_cred = g_slurm_auth_create(AUTH_DEFAULT_INDEX, auth_info);
-	xfree(auth_info);
-	if (auth_cred == NULL) {
-		error("Creating authentication credential: %m");
-		slurm_seterrno(SLURM_PROTOCOL_AUTHENTICATION_ERROR);
-		goto fail1;
-	}
-
-	/*
-	 * Pack the auth credential.
-	 * Always send SLURM_MIN_PROTOCOL_VERSION since we don't know the
-	 * version at the moment.
-	 */
-	rc = g_slurm_auth_pack(auth_cred, buffer, SLURM_MIN_PROTOCOL_VERSION);
-	(void) g_slurm_auth_destroy(auth_cred);
-	if (rc) {
-		error("Packing authentication credential: %m");
-		slurm_seterrno(SLURM_PROTOCOL_AUTHENTICATION_ERROR);
-		goto fail1;
-	}
-
-	/* Connect to the step */
-	fd = _step_connect(directory, nodename, jobid, stepid);
-	if (fd == -1)
-		goto fail1;
-
-	safe_write(fd, &req, sizeof(int));
-	len = size_buf(buffer);
-	safe_write(fd, &len, sizeof(int));
-	safe_write(fd, get_buf_data(buffer), len);
-
-	safe_read(fd, &rc, sizeof(int));
-	if (rc < 0) {
-		error("slurmstepd refused authentication: %m");
-		slurm_seterrno(SLURM_PROTOCOL_AUTHENTICATION_ERROR);
-		goto rwfail;
-	} else if (rc) {
-		*protocol_version = rc;
-	}
-
-	free_buf(buffer);
-	xfree(local_nodename);
-	return fd;
-
-rwfail:
-	close(fd);
-fail1:
-	free_buf(buffer);
-	xfree(local_nodename);
-	return -1;
-}
-
 /*
  * Connect to a slurmstepd proccess by way of its unix domain socket.
  *
@@ -308,7 +249,7 @@ fail1:
  * protocol_version for the stepd, or -1 on error.
  */
 extern int stepd_connect(const char *directory, const char *nodename,
-			 uint32_t jobid, uint32_t stepid,
+			 slurm_step_id_t *step_id,
 			 uint16_t *protocol_version)
 {
 	int req = SLURM_PROTOCOL_VERSION;
@@ -324,14 +265,14 @@ extern int stepd_connect(const char *directory, const char *nodename,
 		nodename = local_nodename;
 	}
 	if (directory == NULL) {
-		slurm_ctl_conf_t *cf = slurm_conf_lock();
+		slurm_conf_t *cf = slurm_conf_lock();
 		directory = slurm_conf_expand_slurmd_path(cf->slurmd_spooldir,
 							  nodename);
 		slurm_conf_unlock();
 	}
 
 	/* Connect to the step */
-	fd = _step_connect(directory, nodename, jobid, stepid);
+	fd = _step_connect(directory, nodename, step_id);
 	if (fd == -1)
 		goto fail1;
 
@@ -347,64 +288,11 @@ extern int stepd_connect(const char *directory, const char *nodename,
 
 rwfail:
 	close(fd);
-	/*
-	 * Most likely case for ending up here is when connecting to a
-	 * pre-19.05 stepd. Assume that the stepd shut the connection down
-	 * since we sent SLURM_PROTOCOL_VERSION instead of SOCKET_CONNECT,
-	 * and retry with the older connection style. Remove this fallback
-	 * 2 versions after 19.05.
-	 */
-	fd = _stepd_connect_legacy(directory, nodename, jobid, stepid,
-				   protocol_version);
 fail1:
 	xfree(local_nodename);
 	return fd;
 }
 
-/*
- * Connect to a slurmstepd proccess by way of its unix domain socket.
- *
- * This is specifically intended to be used with nss_slurm to prevent possible
- * deadlocks. Neither "directory" or "nodename" may be null, and will result
- * in an error. Remove this function in 20.11.
- *
- * Returns a file descriptor for the opened socket on success alongside the
- * protocol_version for the stepd, or -1 on error.
- */
-extern int stepd_connect_nss(const char *directory, const char *nodename,
-			     uint32_t jobid, uint32_t stepid,
-			     uint16_t *protocol_version)
-{
-	int req = SLURM_PROTOCOL_VERSION;
-	int fd = -1;
-	int rc;
-
-	*protocol_version = 0;
-
-	if (!nodename || !directory) {
-		error("directory or nodename invalid");
-		return -1;
-	}
-
-	/* Connect to the step */
-	fd = _step_connect(directory, nodename, jobid, stepid);
-	if (fd == -1)
-		goto fail1;
-
-	safe_write(fd, &req, sizeof(int));
-	safe_read(fd, &rc, sizeof(int));
-	if (rc < 0)
-		goto rwfail;
-	else if (rc)
-		*protocol_version = rc;
-
-	return fd;
-
-rwfail:
-	close(fd);
-fail1:
-	return fd;
-}
 
 /*
  * Retrieve a job step's current state.
@@ -434,14 +322,21 @@ slurmstepd_info_t *stepd_get_info(int fd)
 	safe_write(fd, &req, sizeof(int));
 
 	safe_read(fd, &step_info->uid, sizeof(uid_t));
-	safe_read(fd, &step_info->jobid, sizeof(uint32_t));
-	safe_read(fd, &step_info->stepid, sizeof(uint32_t));
+	safe_read(fd, &step_info->step_id.job_id, sizeof(uint32_t));
+	safe_read(fd, &step_info->step_id.step_id, sizeof(uint32_t));
 
 	safe_read(fd, &step_info->protocol_version, sizeof(uint16_t));
-	if (step_info->protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (step_info->protocol_version >= SLURM_20_11_PROTOCOL_VERSION) {
 		safe_read(fd, &step_info->nodeid, sizeof(uint32_t));
 		safe_read(fd, &step_info->job_mem_limit, sizeof(uint64_t));
 		safe_read(fd, &step_info->step_mem_limit, sizeof(uint64_t));
+		safe_read(fd, &step_info->step_id.step_het_comp,
+			  sizeof(uint32_t));
+	} else if (step_info->protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		safe_read(fd, &step_info->nodeid, sizeof(uint32_t));
+		safe_read(fd, &step_info->job_mem_limit, sizeof(uint64_t));
+		safe_read(fd, &step_info->step_mem_limit, sizeof(uint64_t));
+		step_info->step_id.step_het_comp = NO_VAL;
 	} else {
 		error("%s: protocol_version %hu not supported",
 		      __func__, step_info->protocol_version);
@@ -513,6 +408,35 @@ rwfail:
 	return -1;
 }
 
+/*
+ * Request to enter namespace of a job
+ * -1 on error;
+ */
+extern int stepd_get_namespace_fd(int fd, uint16_t protocol_version)
+{
+	int req = REQUEST_GET_NS_FD;
+	int ns_fd = 0;
+
+	debug("entering %s", __func__);
+	safe_write(fd, &req, sizeof(int));
+
+	safe_read(fd, &ns_fd, sizeof(ns_fd));
+
+	/*
+	 * Receive the file descriptor of the namespace to be joined if valid fd
+	 * is coming. Note that the number of ns_fd will not be the same
+	 * returned from receive_fd_over_pipe().  The number we got from the
+	 * safe_read was the fd on the sender which will be different on our
+	 * end.
+	 */
+	if (ns_fd > 0)
+		ns_fd = receive_fd_over_pipe(fd);
+
+	return ns_fd;
+
+rwfail:
+	return -1;
+}
 
 /*
  * Attach a client to a running job step.
@@ -585,7 +509,8 @@ _sockname_regex_init(regex_t *re, const char *nodename)
 
 	xstrcat(pattern, "^");
 	xstrcat(pattern, nodename);
-	xstrcat(pattern, "_([[:digit:]]*)\\.([[:digit:]]*)$");
+	xstrcat(pattern,
+		"_([[:digit:]]*)\\.([[:digit:]]*)\\.{0,1}([[:digit:]]*)$");
 
 	if (regcomp(re, pattern, REG_EXTENDED) != 0) {
 		error("sockname regex compilation failed");
@@ -598,12 +523,14 @@ _sockname_regex_init(regex_t *re, const char *nodename)
 }
 
 static int
-_sockname_regex(regex_t *re, const char *filename,
-		uint32_t *jobid, uint32_t *stepid)
+_sockname_regex(regex_t *re, const char *filename, slurm_step_id_t *step_id)
 {
 	size_t nmatch = 5;
 	regmatch_t pmatch[5];
 	char *match;
+	size_t my_size;
+
+	xassert(step_id);
 
 	memset(pmatch, 0, sizeof(regmatch_t)*nmatch);
 	if (regexec(re, filename, nmatch, pmatch, 0) == REG_NOMATCH) {
@@ -612,14 +539,22 @@ _sockname_regex(regex_t *re, const char *filename,
 
 	match = xstrndup(filename + pmatch[1].rm_so,
 			(size_t)(pmatch[1].rm_eo - pmatch[1].rm_so));
-	*jobid = (uint32_t)atoll(match);
+	step_id->job_id = slurm_atoul(match);
 	xfree(match);
 
 	match = xstrndup(filename + pmatch[2].rm_so,
 			(size_t)(pmatch[2].rm_eo - pmatch[2].rm_so));
-	*stepid = (uint32_t)atoll(match);
+	step_id->step_id = slurm_atoul(match);
 	xfree(match);
 
+	/* If we have a size here we have a het_comp */
+	if ((my_size = pmatch[3].rm_eo - pmatch[3].rm_so)) {
+		match = xstrndup(filename + pmatch[3].rm_so, my_size);
+		step_id->step_het_comp = slurm_atoul(match);
+		xfree(match);
+	} else
+		step_id->step_het_comp = NO_VAL;
+
 	return 0;
 }
 
@@ -650,9 +585,7 @@ stepd_available(const char *directory, const char *nodename)
 		}
 	}
 	if (directory == NULL) {
-		slurm_ctl_conf_t *cf;
-
-		cf = slurm_conf_lock();
+		slurm_conf_t *cf = slurm_conf_lock();
 		directory = slurm_conf_expand_slurmd_path(
 			cf->slurmd_spooldir, nodename);
 		slurm_conf_unlock();
@@ -680,15 +613,14 @@ stepd_available(const char *directory, const char *nodename)
 
 	while ((ent = readdir(dp)) != NULL) {
 		step_loc_t *loc;
-		uint32_t jobid, stepid;
+		slurm_step_id_t step_id;
 
-		if (_sockname_regex(&re, ent->d_name, &jobid, &stepid) == 0) {
-			debug4("found jobid = %u, stepid = %u", jobid, stepid);
+		if (!_sockname_regex(&re, ent->d_name, &step_id)) {
+			debug4("found %ps", &step_id);
 			loc = xmalloc(sizeof(step_loc_t));
 			loc->directory = xstrdup(directory);
 			loc->nodename = xstrdup(nodename);
-			loc->jobid = jobid;
-			loc->stepid = stepid;
+			memcpy(&loc->step_id, &step_id, sizeof(loc->step_id));
 			list_append(l, (void *)loc);
 		}
 	}
@@ -732,21 +664,22 @@ stepd_cleanup_sockets(const char *directory, const char *nodename)
 	}
 
 	while ((ent = readdir(dp)) != NULL) {
-		uint32_t jobid, stepid;
-		if (_sockname_regex(&re, ent->d_name, &jobid, &stepid) == 0) {
+		slurm_step_id_t step_id;
+		if (!_sockname_regex(&re, ent->d_name, &step_id)) {
 			char *path;
 			int fd;
 			uint16_t protocol_version;
 
 			path = NULL;
 			xstrfmtcat(path, "%s/%s", directory, ent->d_name);
-			verbose("Cleaning up stray job step %u.%u",
-				jobid, stepid);
+
+			verbose("Cleaning up stray %ps", &step_id);
 
 			/* signal the slurmstepd to terminate its step */
 			fd = stepd_connect((char *) directory,
 					   (char *) nodename,
-					   jobid, stepid, &protocol_version);
+					   &step_id,
+					   &protocol_version);
 			if (fd == -1) {
 				debug("Unable to connect to socket %s", path);
 			} else {
@@ -754,8 +687,8 @@ stepd_cleanup_sockets(const char *directory, const char *nodename)
 					    fd, protocol_version, SIGKILL, 0,
 					    getuid())
 				    == -1) {
-					debug("Error sending SIGKILL to job step %u.%u",
-					      jobid, stepid);
+					debug("Error sending SIGKILL to %ps",
+					      &step_id);
 				}
 				close(fd);
 			}
@@ -1144,9 +1077,8 @@ stepd_completion(int fd, uint16_t protocol_version, step_complete_msg_t *sent)
 
 	buffer = init_buf(0);
 
-	debug("Entering stepd_completion for %u.%u, range_first = %d, range_last = %d",
-	      sent->job_id, sent->job_step_id,
-	      sent->range_first, sent->range_last);
+	debug("Entering stepd_completion for %ps, range_first = %d, range_last = %d",
+	      &sent->step_id, sent->range_first, sent->range_last);
 
 	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_write(fd, &req, sizeof(int));
@@ -1194,7 +1126,7 @@ rwfail:
  */
 int
 stepd_stat_jobacct(int fd, uint16_t protocol_version,
-		   job_step_id_msg_t *sent, job_step_stat_t *resp)
+		   slurm_step_id_t *sent, job_step_stat_t *resp)
 {
 	int req = REQUEST_STEP_STAT;
 	int rc = SLURM_SUCCESS;
@@ -1204,8 +1136,7 @@ stepd_stat_jobacct(int fd, uint16_t protocol_version,
 	if (!(resp->jobacct = jobacctinfo_create(NULL)))
 		return rc;
 
-	debug("Entering stepd_stat_jobacct for job %u.%u",
-	      sent->job_id, sent->step_id);
+	debug("Entering %s for %ps", __func__, sent);
 
 	safe_write(fd, &req, sizeof(int));
 
