@@ -43,6 +43,7 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <sys/mman.h>
+#include <sys/stat.h>
 #include <unistd.h>
 
 #include "src/common/assoc_mgr.h"
@@ -88,6 +89,8 @@ static stepd_step_rec_t *_step_setup(slurm_addr_t *cli, slurm_addr_t *self,
 static void _step_cleanup(stepd_step_rec_t *job, slurm_msg_t *msg, int rc);
 #endif
 static int _process_cmdline (int argc, char **argv);
+static void _remove_spool_job_dir(stepd_step_rec_t *job);
+static void _remove_spool_sub_dir(const char *directory);
 
 /*
  *  List of signals to block in this process
@@ -111,11 +114,12 @@ main (int argc, char **argv)
 	slurm_msg_t *msg;
 	stepd_step_rec_t *job;
 	int rc = 0;
-	char *launch_params;
 
 	if (_process_cmdline (argc, argv) < 0)
 		fatal ("Error in slurmstepd command line");
 
+	slurm_conf_init(NULL);
+
 	xsignal_block(slurmstepd_blocked_signals);
 	conf = xmalloc(sizeof(*conf));
 	conf->argv = &argv;
@@ -151,18 +155,17 @@ main (int argc, char **argv)
 		goto ending;
 	}
 
-	if (job->stepid != SLURM_EXTERN_CONT)
+	if (job->step_id.step_id != SLURM_EXTERN_CONT)
 		close_slurmd_conn();
 
 	/* slurmstepd is the only daemon that should survive upgrade. If it
 	 * had been swapped out before upgrade happened it could easily lead
 	 * to SIGBUS at any time after upgrade. Avoid that by locking it
 	 * in-memory. */
-	launch_params = slurm_get_launch_params();
-	if (launch_params && strstr(launch_params, "slurmstepd_memlock")) {
+	if (xstrstr(slurm_conf.launch_params, "slurmstepd_memlock")) {
 #ifdef _POSIX_MEMLOCK
 		int flags = MCL_CURRENT;
-		if (strstr(launch_params, "slurmstepd_memlock_all"))
+		if (xstrstr(slurm_conf.launch_params, "slurmstepd_memlock_all"))
 			flags |= MCL_FUTURE;
 		if (mlockall(flags) < 0)
 			info("failed to mlock() slurmstepd pages: %m");
@@ -172,7 +175,6 @@ main (int argc, char **argv)
 		info("mlockall() system call does not appear to be available");
 #endif
 	}
-	xfree(launch_params);
 
 	acct_gather_energy_g_set_data(ENERGY_DATA_STEP_PTR, job);
 
@@ -203,6 +205,8 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *job,
 	if (conf->hwloc_xml)
 		(void)remove(conf->hwloc_xml);
 
+	_remove_spool_job_dir(job);
+
 #ifdef MEMORY_LEAK_DEBUG
 	acct_gather_conf_destroy();
 	(void) core_spec_g_fini();
@@ -218,15 +222,11 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *job,
 	xfree(conf->block_map_inv);
 	xfree(conf->hostname);
 	xfree(conf->hwloc_xml);
-	xfree(conf->job_acct_gather_freq);
-	xfree(conf->job_acct_gather_type);
 	xfree(conf->logfile);
 	xfree(conf->node_name);
 	xfree(conf->node_topo_addr);
 	xfree(conf->node_topo_pattern);
 	xfree(conf->spooldir);
-	xfree(conf->task_epilog);
-	xfree(conf->task_prolog);
 	xfree(conf);
 #endif
 	info("done with job");
@@ -278,10 +278,12 @@ static slurmd_conf_t *read_slurmd_conf_lite(int fd)
 	if (rc == SLURM_ERROR)
 		fatal("slurmstepd: problem with unpack of slurmd_conf");
 
-	slurm_unpack_list(&tmp_list,
-			  slurmdb_unpack_tres_rec,
-			  slurmdb_destroy_tres_rec,
-			  buffer, SLURM_PROTOCOL_VERSION);
+	if (slurm_unpack_list(&tmp_list,
+			      slurmdb_unpack_tres_rec,
+			      slurmdb_destroy_tres_rec,
+			      buffer, SLURM_PROTOCOL_VERSION)
+	    != SLURM_SUCCESS)
+		fatal("slurmstepd: problem with unpack of tres list");
 
 	free_buf(buffer);
 
@@ -308,12 +310,12 @@ static slurmd_conf_t *read_slurmd_conf_lite(int fd)
 	 * up in the log.
 	 */
 	log_alter(confl->log_opts, 0, confl->logfile);
-	log_set_timefmt(confl->log_fmt);
-	debug2("debug level is %d.", confl->debug_level);
+	log_set_timefmt(slurm_conf.log_fmt);
+	debug2("debug level is '%s'.", log_num2string(confl->debug_level));
 
 	confl->acct_freq_task = NO_VAL16;
 	tmp_int = acct_gather_parse_freq(PROFILE_TASK,
-				       confl->job_acct_gather_freq);
+					 slurm_conf.job_acct_gather_freq);
 	if (tmp_int != -1)
 		confl->acct_freq_task = tmp_int;
 
@@ -483,19 +485,16 @@ rwfail:
 #endif
 }
 
-static void _set_job_log_prefix(uint32_t jobid, uint32_t stepid)
+static void _set_job_log_prefix(slurm_step_id_t *step_id)
 {
 	char *buf;
+	char tmp_char[64];
 
-	if (stepid == SLURM_BATCH_SCRIPT)
-		buf = xstrdup_printf("[%u.batch]", jobid);
-	else if (stepid == SLURM_EXTERN_CONT)
-		buf = xstrdup_printf("[%u.extern]", jobid);
-	else
-		buf = xstrdup_printf("[%u.%u]", jobid, stepid);
+	log_build_step_id_str(step_id, tmp_char, sizeof(tmp_char),
+			      STEP_ID_FLAG_NO_PREFIX);
+	buf = xstrdup_printf("[%s]", tmp_char);
 
 	setproctitle("%s", buf);
-
 	/* note: will claim ownership of buf, do not free */
 	xstrcat(buf, " ");
 	log_set_fpfx(&buf);
@@ -517,9 +516,11 @@ _init_from_slurmd(int sock, char **argv,
 	slurm_addr_t *cli = NULL;
 	slurm_addr_t *self = NULL;
 	slurm_msg_t *msg = NULL;
-	uint16_t port;
-	char buf[16];
-	uint32_t jobid = 0, stepid = 0;
+	slurm_step_id_t step_id = {
+		.job_id = 0,
+		.step_id = NO_VAL,
+		.step_het_comp = NO_VAL,
+	};
 
 	/* receive conf from slurmd */
 	if (!(conf = read_slurmd_conf_lite(sock)))
@@ -552,9 +553,8 @@ _init_from_slurmd(int sock, char **argv,
 
 	switch_g_slurmd_step_init();
 
-	slurm_get_ip_str(&step_complete.parent_addr, &port, buf, 16);
-	debug3("slurmstepd rank %d, parent address = %s, port = %u",
-	       step_complete.rank, buf, port);
+	debug3("slurmstepd rank %d, parent = %pA",
+	       step_complete.rank, &step_complete.parent_addr);
 
 	/* receive cli from slurmd */
 	safe_read(sock, &len, sizeof(int));
@@ -562,7 +562,7 @@ _init_from_slurmd(int sock, char **argv,
 	safe_read(sock, incoming_buffer, len);
 	buffer = create_buf(incoming_buffer,len);
 	cli = xmalloc(sizeof(slurm_addr_t));
-	if (slurm_unpack_slurm_addr_no_alloc(cli, buffer) == SLURM_ERROR)
+	if (slurm_unpack_addr_no_alloc(cli, buffer) == SLURM_ERROR)
 		fatal("slurmstepd: problem with unpack of slurmd_conf");
 	free_buf(buffer);
 
@@ -574,7 +574,7 @@ _init_from_slurmd(int sock, char **argv,
 		safe_read(sock, incoming_buffer, len);
 		buffer = create_buf(incoming_buffer,len);
 		self = xmalloc(sizeof(slurm_addr_t));
-		if (slurm_unpack_slurm_addr_no_alloc(self, buffer)
+		if (slurm_unpack_addr_no_alloc(self, buffer)
 		    == SLURM_ERROR) {
 			fatal("slurmstepd: problem with unpack of "
 			      "slurmd_conf");
@@ -582,9 +582,6 @@ _init_from_slurmd(int sock, char **argv,
 		free_buf(buffer);
 	}
 
-	/* Receive GRES information from slurmd */
-	gres_plugin_recv_stepd(sock);
-
 	/* Grab the slurmd's spooldir. Has %n expanded. */
 	cpu_freq_init(conf);
 
@@ -622,25 +619,43 @@ _init_from_slurmd(int sock, char **argv,
 
 	switch (step_type) {
 	case LAUNCH_BATCH_JOB:
-		jobid = ((batch_job_launch_msg_t *)msg->data)->job_id;
-		stepid = ((batch_job_launch_msg_t *)msg->data)->step_id;
+		step_id.job_id = ((batch_job_launch_msg_t *)msg->data)->job_id;
+		step_id.step_id = SLURM_BATCH_SCRIPT;
+		step_id.step_het_comp = NO_VAL;
 		break;
 	case LAUNCH_TASKS:
-		jobid = ((launch_tasks_request_msg_t *)msg->data)->job_id;
-		stepid = ((launch_tasks_request_msg_t *)msg->data)->job_step_id;
+		memcpy(&step_id,
+		       &((launch_tasks_request_msg_t *)msg->data)->step_id,
+		       sizeof(step_id));
 		break;
 	default:
 		fatal("%s: Unrecognized launch RPC (%d)", __func__, step_type);
 		break;
 	}
 
-	_set_job_log_prefix(jobid, stepid);
+	/* Receive GRES information from slurmd */
+	gres_plugin_recv_stepd(sock, msg);
 
-	if (!conf->hwloc_xml)
-		conf->hwloc_xml = xstrdup_printf("%s/hwloc_topo_%u.%u.xml",
+	/*
+	 * Read slurmd node name - may be dictated by slurmctld due to
+	 * Dynamic Future node mapping.
+	 */
+	safe_read(sock, &len, sizeof(int));
+	conf->node_name = xmalloc(len);
+	safe_read(sock, conf->node_name, len);
+
+	_set_job_log_prefix(&step_id);
+
+	if (!conf->hwloc_xml) {
+		conf->hwloc_xml = xstrdup_printf("%s/hwloc_topo_%u.%u",
 						 conf->spooldir,
-						 jobid, stepid);
-
+						 step_id.job_id,
+						 step_id.step_id);
+		if (step_id.step_het_comp != NO_VAL)
+			xstrfmtcat(conf->hwloc_xml, ".%u",
+				   step_id.step_het_comp);
+		xstrcat(conf->hwloc_xml, ".xml");
+	}
 	/*
 	 * Swap the field to the srun client version, which will eventually
 	 * end up stored as protocol_version in srun_info_t. It's a hack to
@@ -689,10 +704,12 @@ _step_setup(slurm_addr_t *cli, slurm_addr_t *self, slurm_msg_t *msg)
 	job->jobacct = jobacctinfo_create(NULL);
 
 	/* Establish GRES environment variables */
-	if (conf->debug_flags & DEBUG_FLAG_GRES) {
-		gres_plugin_job_state_log(job->job_gres_list, job->jobid);
-		gres_plugin_step_state_log(job->step_gres_list, job->jobid,
-					   job->stepid);
+	if (slurm_conf.debug_flags & DEBUG_FLAG_GRES) {
+		gres_plugin_job_state_log(job->job_gres_list,
+					  job->step_id.job_id);
+		gres_plugin_step_state_log(job->step_gres_list,
+					   job->step_id.job_id,
+					   job->step_id.step_id);
 	}
 	if (msg->msg_type == REQUEST_BATCH_JOB_LAUNCH) {
 		gres_plugin_job_set_env(&job->env, job->job_gres_list, 0);
@@ -754,3 +771,79 @@ static void _dump_user_env(void)
 	for (i=0; environ[i]; i++)
 		printf("%s\n",environ[i]);
 }
+
+static void
+_remove_spool_job_dir(stepd_step_rec_t *job)
+{
+	char *job_id = NULL, *directory = NULL;
+
+	if (!job->env)
+		return;
+	job_id = xstrdup(getenvp(job->env, "SLURM_JOB_ID"));
+	if (job_id) {
+		xstrfmtcat(directory, "%s/%s", conf->spooldir, job_id);
+		debug("%s: conf->spooldir: %s", __func__, conf->spooldir);
+		debug("%s: job_id: %s", __func__, job_id);
+		debug("%s: directory: %s", __func__, directory);
+		_remove_spool_sub_dir(directory);
+		xfree(directory);
+		xfree(job_id);
+	}
+}
+
+static void
+_remove_spool_sub_dir(const char *directory)
+{
+	const char *spool = "/var/spool/";
+	char *strdir = NULL, *startp = NULL, *prefix = NULL;
+	size_t len1 = 0, len2 = 0;
+	struct stat stat_buf;
+	DIR *dp;
+	struct dirent *ent;
+	char *pathname = NULL;
+
+	if (!directory)
+		return;
+
+	/*
+	 * Don't remove to any directory other than slurmd spool sub directory.
+	 * (The default value is "/var/spool/slurmd/<sub_dir>[/<sub_dir>...]".)
+	 */
+	if (xstrncmp(conf->spooldir, spool, strlen(spool)) != 0)
+		return;
+	len1 = strlen(conf->spooldir);
+	if (xstrncmp(directory, conf->spooldir, len1) != 0)
+		return;
+	strdir = xstrdup(directory);
+	startp = (strdir + len1);
+	while ((prefix = xstrchr(startp, '/'))) {
+		len2 += 1;
+		startp = (prefix + 1);
+	}
+	xfree(strdir);
+	if (strlen(directory) <= (len1 + len2))
+		return;
+
+	if (stat(directory, &stat_buf) < 0)
+		return;
+	if (!S_ISDIR(stat_buf.st_mode))
+		return;
+	if ((dp = opendir(directory)) == NULL)
+		return;
+	while ((ent = readdir(dp)) != NULL) {
+		if (xstrcmp(ent->d_name, ".") == 0)
+			continue;
+		if (xstrcmp(ent->d_name, "..") == 0)
+			continue;
+		xstrfmtcat(pathname, "%s/%s", directory, ent->d_name);
+		stat(pathname, &stat_buf);
+		if (S_ISDIR(stat_buf.st_mode)) {
+			if (rmdir(pathname) < 0)
+				_remove_spool_sub_dir(pathname);
+		} else
+			(void)unlink(pathname);
+		xfree(pathname);
+	}
+	closedir(dp);
+	(void)rmdir(directory);
+}
