@@ -86,7 +86,7 @@ START_TEST(test_data_job_macros)
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_CONTIGUOUS, arg,
 						errors) == 0,
 		      "LONG_OPT_CONTIGUOUS=true");
-	ck_assert_msg(opt.contiguous == true, "contiguous=true");
+	ck_assert_msg(opt.contiguous, true, "contiguous=true");
 	data_set_bool(arg, false);
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_CONTIGUOUS, arg,
 						errors) == 0,
@@ -97,7 +97,7 @@ START_TEST(test_data_job_macros)
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_CPUS_PER_GPU,
 						arg, errors) == 0,
 		      "LONG_OPT_CPUS_PER_GPU");
-	ck_assert_msg(opt.cpus_per_gpu == 12345, "cpus_per_gpu");
+	ck_assert_msg(opt.cpus_per_gpu, 12345, "cpus_per_gpu");
 	data_set_string(arg, "0");
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_CPUS_PER_GPU,
 						arg, errors) == 0,
@@ -175,7 +175,7 @@ START_TEST(test_data_job)
 		      "core spec nochange");
 
 	/* force enable all governors */
-	slurmctld_conf.cpu_freq_govs = 0xffffffff;
+	slurm_conf.cpu_freq_govs = 0xffffffff;
 	data_set_string(arg, "10-100:PowerSave");
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_CPU_FREQ, arg,
 						errors) == 0, "cpu freq");
@@ -325,10 +325,10 @@ START_TEST(test_data_job)
 		      != 0, "gid");
 	ck_assert_msg(opt.gid == NO_VAL, "gid value");
 	/* verify that group of slurmuser can be used */
-	data_set_string(arg, slurmctld_conf.slurm_user_name);
+	data_set_string(arg, slurm_conf.slurm_user_name);
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_GID, arg, errors)
 		      == 0, "gid");
-	ck_assert_msg(opt.gid == gid_from_uid(slurmctld_conf.slurm_user_id),
+	ck_assert_msg(opt.gid == gid_from_uid(slurm_conf.slurm_user_id),
 		      "gid value");
 
 	data_set_string(arg, "help");
@@ -516,17 +516,11 @@ START_TEST(test_data_job)
 	data_set_string(arg, "append");
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_OPEN_MODE, arg,
 						errors) == 0, "open mode");
-	ck_assert_msg(opt.sbatch_opt->open_mode == OPEN_MODE_APPEND,
-		      "open mode value");
-	ck_assert_msg(opt.srun_opt->open_mode == OPEN_MODE_APPEND,
-		      "open mode value");
+	ck_assert_msg(opt.open_mode == OPEN_MODE_APPEND, "open mode value");
 	data_set_string(arg, "truncate");
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_OPEN_MODE, arg,
 						errors) == 0, "open mode");
-	ck_assert_msg(opt.sbatch_opt->open_mode == OPEN_MODE_TRUNCATE,
-		      "open mode value");
-	ck_assert_msg(opt.srun_opt->open_mode == OPEN_MODE_TRUNCATE,
-		      "open mode value");
+	ck_assert_msg(opt.open_mode == OPEN_MODE_TRUNCATE, "open mode value");
 	data_set_string(arg, "invalid");
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_OPEN_MODE, arg,
 						errors) != 0, "open mode");
@@ -657,7 +651,7 @@ START_TEST(test_data_job)
 						errors) != 0, "thread-spec");
 
 	data_set_string(arg, "invalid");
-	ck_assert_msg(slurm_process_option_data(&opt, 't', arg, errors) != 0,
+	ck_assert_msg(slurm_process_option_data(&opt, 't', arg, errors), 0,
 		      "time-limit");
 	data_set_string(arg, "0");
 	ck_assert_msg(slurm_process_option_data(&opt, 't', arg, errors) == 0,
@@ -686,10 +680,10 @@ START_TEST(test_data_job)
 		      != 0, "gid");
 	ck_assert_msg(opt.uid == NO_VAL, "uid value");
 	/* verify that slurmuser can be used */
-	data_set_string(arg, slurmctld_conf.slurm_user_name);
+	data_set_string(arg, slurm_conf.slurm_user_name);
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_UID, arg, errors)
 		      == 0, "uid");
-	ck_assert_msg(opt.uid == slurmctld_conf.slurm_user_id, "uid value");
+	ck_assert_msg(opt.uid == slurm_conf.slurm_user_id, "uid value");
 
 	data_set_string(arg, "invalid");
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_UMASK, arg,
